﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports System.Windows.Navigation
Imports System.Windows.Ria
Imports System.Windows.Ria.ApplicationServices
Imports System.Windows.Ria.Data
Imports System.Windows.Data
Imports System.ComponentModel
Imports System.Windows.Threading
Imports AlexApp

Namespace ExpenseReport
	Partial Public Class ExpenseReportSummary
		Inherits Page
		Public Sub New()
			InitializeComponent()

			AddHandler ddsYourExpenseReports.SubmittedChanges, AddressOf ddsYourExpenseReports_SubmittedChanges
			AddHandler ddsApprovalExpenseReports.SubmittedChanges, AddressOf ddsYourExpenseReports_SubmittedChanges
			ProcessLogin()
		End Sub

		''' <summary>
		''' Method is required to allow refreshing of DomainDataSource after an Submit\Approve\Reject has 
		''' been submitted.
		''' </summary>
		''' <param name="sender">DomainDataSource that submitted changes</param>
		''' <param name="e">SubmittedChangesEventArgs returned by event</param>
		Private Sub ddsYourExpenseReports_SubmittedChanges(ByVal sender As Object, ByVal e As SubmittedChangesEventArgs)
			ProcessLogin()
		End Sub

		''' <summary>
		''' Validates that the current user is authenticated and then explictly loads the correct 
		''' datasources based on the users role.
		''' </summary>
		Private Sub ProcessLogin()
			Dim currentUser As User = TryCast(UserService.Current.User, User)
			If currentUser IsNot Nothing AndAlso currentUser.IsAuthenticated Then
				If currentUser.Roles.Contains("employee") Then
					ddsYourExpenseReports.Load()
					yourExpenseReportPanel.Visibility = Visibility.Visible
					ddsYourExpenseReports.AutoLoad = True
				End If
				If currentUser.Roles.Contains("manager") Then
					ddsApprovalExpenseReports.Load()
					approvalExpenseReportPanel.Visibility = Visibility.Visible
					ddsApprovalExpenseReports.AutoLoad = True
				End If
			Else
				yourExpenseReportPanel.Visibility = Visibility.Collapsed
				approvalExpenseReportPanel.Visibility = Visibility.Collapsed
				ddsYourExpenseReports.AutoLoad = False
				ddsApprovalExpenseReports.AutoLoad = False
			End If
		End Sub

		''' <summary>
		''' Navigates to UpdateExpenseReport page with a zero ID so that a new expense report
		''' details is created.
		''' </summary>
		''' <param name="sender">Button initiating event.</param>
		''' <param name="e">RoutedEventArgs for button click</param>
		Private Sub CreateButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			Dim uriString As String = "/UpdateExpenseReport.xaml?ExpenseReportId=" & "0"
			Me.NavigationService.Navigate(New Uri(uriString, UriKind.Relative))
		End Sub

		''' <summary>
		''' Navigates to UpdateExpenseReport page with a ID of selected expense report 
		''' so that details of expense report are shown on new page.
		''' </summary>
		''' <param name="sender">Button initiating event.</param>
		''' <param name="e">RoutedEventArgs for button click</param>
		Private Sub EditButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			If dgYourExpenseReport.SelectedIndex >= 0 Then
				Dim uriString As String = "/UpdateExpenseReport.xaml?ExpenseReportId=" & (CType(dgYourExpenseReport.SelectedItem, ExpenseReport)).id
				Me.NavigationService.Navigate(New Uri(uriString, UriKind.Relative))
			End If
		End Sub

		''' <summary>
		''' Submits the expense report and saves the changes to the database.
		''' </summary>
		''' <param name="sender">Button initiating event.</param>
		''' <param name="e">RoutedEventArgs for button click</param>
		Private Sub SubmitButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			If dgYourExpenseReport.SelectedIndex >= 0 Then
				CType(ddsYourExpenseReports.DomainContext, ExpenseData).SubmitExpenseReport(CType(dgYourExpenseReport.SelectedItem, ExpenseReport))
				ddsYourExpenseReports.SubmitChanges()
			End If
		End Sub

		''' <summary>
		''' Approves the expense report and saves the changes to the database.
		''' </summary>
		''' <param name="sender">Button initiating event.</param>
		''' <param name="e">RoutedEventArgs for button click</param>
		Private Sub ApproveButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			If dgApprovalExpenseReport.SelectedIndex >= 0 Then
				CType(ddsYourExpenseReports.DomainContext, ExpenseData).ApproveExpenseReport(CType(dgYourExpenseReport.SelectedItem, ExpenseReport))
				ddsApprovalExpenseReports.SubmitChanges()
			End If
		End Sub

		''' <summary>
		''' Rejects the expense report and saves the changes to the database.
		''' </summary>
		''' <param name="sender">Button initiating event.</param>
		''' <param name="e">RoutedEventArgs for button click</param>
		Private Sub RejectButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			If dgApprovalExpenseReport.SelectedIndex >= 0 Then
				CType(ddsYourExpenseReports.DomainContext, ExpenseData).RejectExpenseReport(CType(dgYourExpenseReport.SelectedItem, ExpenseReport))
				ddsApprovalExpenseReports.SubmitChanges()
			End If
		End Sub

	End Class
End Namespace
