﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Collections
Imports System.ComponentModel
Imports System.Linq
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Shapes
Imports System.Windows.Navigation
Imports System.Windows.Ria
Imports System.Windows.Data
Imports System.Windows.Ria.ApplicationServices
Imports System.Windows.Ria.Data
Imports AlexApp

Namespace ExpenseReport
	''' <summary>
	''' Allows modification of an ExpenseReport and the individual ExpenseReportDetails via the
	''' ModifyExpenseDetailChildWindow.  All changes to details are batched and not saved until 
	''' the parent item is committed.
	''' </summary>
	Partial Public Class UpdateExpenseReport
		Inherits Page
		Public currentExpenseReport As ExpenseReport = Nothing
		Public currentExpenseReportDetail As ExpenseReportDetails = Nothing

		Private detailWindow As ModifyExpenseDetailChildWindow = Nothing

		Public Sub New()
			InitializeComponent()

			AddHandler expenseForm.ItemEditEnded, AddressOf expenseForm_ItemEditEnded
		End Sub

		''' <summary>
		''' Persists the changes to the server when data form has completed commiting an edit.
		''' </summary>
		Private Sub expenseForm_ItemEditEnded(ByVal sender As Object, ByVal e As DataFormItemEditEndedEventArgs)
			If e.EditAction = DataFormEditAction.Commit Then
				ods.SubmitChanges()
			End If
			expenseForm.BeginEdit()
		End Sub

		''' <summary>
		''' Retrieves the current expense report once the page is loaded.
		''' </summary>
		Private Sub ods_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			'if ExpenseReportId is 0 then this is a create path and there is no need to query
			'just create a new one.
			Dim expenseReportId As Integer = Integer.Parse(Me.NavigationContext.QueryString("ExpenseReportId"))
			If expenseReportId = 0 Then
				Dim currentUser As User = TryCast(UserService.Current.User, User)
				Dim newExpenseReport As New ExpenseReport() With {.RptStatus = ReportStatus.NotSubmitted, .Created=DateTime.Now.Date, .user = currentUser.Name}

				CType(ods.DomainContext, ExpenseData).ExpenseReports.Add(newExpenseReport)
				expenseForm.CurrentItem = newExpenseReport
				currentExpenseReport = newExpenseReport
			Else 'Set the load method parameter
				ods.LoadParameters.Add(New Parameter() With {.ParameterName = "erId", .Value = expenseReportId})
				ods.Load()
			End If
		End Sub

		''' <summary>
		''' Sets the current item for the expense report from.
		''' </summary>
		Private Sub ods_LoadedData(ByVal sender As Object, ByVal e As LoadedDataEventArgs)
			Dim dataSource As DomainDataSource
			If TypeOf sender Is DomainDataSource Then
				dataSource = CType(sender, DomainDataSource)
				If TypeOf dataSource.DataView Is IEnumerable Then
					Dim data As IEnumerable = CType(dataSource.Data, IEnumerable)
					Dim dataEnumerator As IEnumerator = data.GetEnumerator()

					expenseForm.CurrentItem = If(dataEnumerator.MoveNext(), dataEnumerator.Current, Nothing)
					currentExpenseReport = CType(expenseForm.CurrentItem, ExpenseReport)
					expenseForm.BeginEdit()
				End If
			End If
		End Sub

		''' <summary>
		''' Creates the detail window 
		''' </summary>
		Private Sub createModifyExpenseDetailChildWindow()
			If detailWindow Is Nothing Then
				detailWindow = New ModifyExpenseDetailChildWindow()
			End If
		End Sub

		''' <summary>
		''' Creates ChildWindow to edit expense report detail and sets properties on ChildWindow
		''' to indicate parent DomainContext and ExpenseReprot
		''' </summary>
		Private Sub btnEditClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
			createModifyExpenseDetailChildWindow()

			'Set embedded window to point to selected ExpenseReportDetails.
			Dim expenseReportDetail As ExpenseReportDetails = currentExpenseReportDetail
			detailWindow.DataContext = currentExpenseReport.ExpenseReportDetails
			detailWindow.ParentDomainContext = CType(ods.DomainContext, ExpenseData)
			detailWindow.ParentExpenseReport = currentExpenseReport
			detailWindow.DetailItem = expenseReportDetail
			detailWindow.Show()

		End Sub

		''' <summary>
		''' Creates a new expense reprot detail and sets the current item on the ChildWindow to point to it.
		''' </summary>
		Private Sub btnAddClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
			'Create new instance and add it to list.
			createModifyExpenseDetailChildWindow()
			Dim newExpenseReportDetail As New ExpenseReportDetails() With {.DateIncurred = DateTime.Now.Date}
			currentExpenseReportDetail = newExpenseReportDetail
			detailWindow.DataContext = newExpenseReportDetail
			detailWindow.ParentDomainContext = CType(ods.DomainContext, ExpenseData)
			detailWindow.ParentExpenseReport = currentExpenseReport
			detailWindow.DetailItem = newExpenseReportDetail
			detailWindow.Show()
		End Sub

		''' <summary>
		''' Sets the current expense report detail when a datagrid selection changes.
		''' </summary>
		Private Sub editDetailsDataGrid_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
			If e.AddedItems.Count > 0 Then
				currentExpenseReportDetail = CType(e.AddedItems(0), ExpenseReportDetails)
			End If
		End Sub

		''' <summary>
		''' Deletes the current expense report detail
		''' </summary>
		Private Sub btnDeleteClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
			currentExpenseReport.ExpenseReportDetails.Remove(currentExpenseReportDetail)
		End Sub

	End Class
End Namespace
