﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.ComponentModel.DataAnnotations
Imports System.Linq
Imports System.Web
Imports System.Web.Ria.Data

Namespace ExpenseReport
    ''' <summary>
    ''' Enumeration representing the possible statuses of expense reports.
    ''' </summary>
    Public Enum ReportStatus
        NotSubmitted = 0
        Submitted = 1
        Approved = 2
    End Enum

    Partial Public Class ExpenseReport
        ''' <summary>
        ''' EntityFramework currently doesn't support enum values so this maps the integer type for status
        ''' into the enumerated values.
        ''' </summary>
        <SharedAttribute(), Bindable(True), Display(ShortName:="Status", Description:="Expense Report Status.")> _
  Public Property RptStatus() As ReportStatus
            Get
                Return CType(Me.Status, ReportStatus)
            End Get
            Set(ByVal value As ReportStatus)
                Me.Status = CInt(Fix(value))
            End Set
        End Property

    End Class

    <SharedAttribute()> _
 Public NotInheritable Class ExpenseReportValidation
        ''' <summary>
        ''' Validation method that checks to make sure creation date isn't greater than today's date.
        ''' </summary>
        ''' <param name="property">The property value being validated.</param>
        ''' <param name="context">The ValidationContext representing the state of the object and property 
        ''' when it is being validated.</param>
        ''' <param name="validationResult">ValidationResult of the validation method including error messages.</param>
        ''' <returns>Boolean indicating whether validation was successful.</returns>
        Private Sub New()
        End Sub
        Public Shared Function ValidateDateIncurred(ByVal [property] As Object, ByVal context As ValidationContext, <System.Runtime.InteropServices.Out()> ByRef validationResult As ValidationResult) As Boolean
            validationResult = Nothing
            Dim result As Boolean = DateTime.Compare(CDate([property]), DateTime.Now) < 0

            If (Not result) Then
                validationResult = New ValidationResult(context.DisplayName & " must be today or in the past.")
            End If
            Return result
        End Function

        ''' <summary>
        ''' Validation method that checks to ensure there are descriptions entered for expenses greater than $1000.
        ''' </summary>
        ''' <param name="property">The property value being validated.</param>
        ''' <param name="context">The ValidationContext representing the state of the object and property 
        ''' when it is being validated.</param>
        ''' <param name="validationResult">ValidationResult of the validation method including error messages.</param>
        ''' <returns>Boolean indicating whether validation was successful.</returns>
        Public Shared Function ValidateDescription(ByVal [property] As Object, ByVal context As ValidationContext, <System.Runtime.InteropServices.Out()> ByRef validationResult As ValidationResult) As Boolean
            validationResult = Nothing
            Dim er As ExpenseReportDetails = CType([property], ExpenseReportDetails)
            Dim result As Boolean
            If er.Amount > 1000 AndAlso (String.IsNullOrEmpty(er.LongDescription) OrElse String.IsNullOrEmpty(er.ShortDescription)) Then
                result = False
            Else
                result = True
            End If

            If (Not result) Then
                validationResult = New ValidationResult("For expenses greater than $1000, summary and description must " & "be completed.")
            End If
            Return result
        End Function
    End Class
End Namespace